package eric

import scala.collection.mutable.{ HashMap, ArrayBuffer }
import cc.factorie._
import cc.factorie.la._
import cc.factorie.variable.{ CategoricalDomain, CategoricalVariable, LabeledCategoricalVariable }
import cc.factorie.model.{ TemplateModel, DotTemplateWithStatistics2, Template2 }
import cc.factorie.infer.{ InferByBPLoopy, GibbsSampler }
import cc.factorie.optimize.{ LikelihoodExample, SampleRankTrainer, AdaGrad, PseudolikelihoodExample }
/**
 * @author eric
 */
object tryProblem {
  
  // Score from predictor
  val pScoreDomain = new CategoricalDomain[String](List.range(0, 10).map(x => x.toString))
  class pScoreVar(val scoreVal: String) extends CategoricalVariable[String](scoreVal) {
    def domain = pScoreDomain
  }
  // variable I'm predicting, connected with pair factors
  val xDomain = new CategoricalDomain[String]
  class xVar(truth: String, val pScore: pScoreVar, val adj: Set[xVar]) extends LabeledCategoricalVariable(truth) { // val adjList: Array[String], val allVar: HashMap[String, xVar]) extends LabeledCategoricalVariable(truth) {
    def domain = xDomain
  }
  // Some variables
  val s1 = new pScoreVar(1.toString)
  val s2 = new pScoreVar(2.toString)
  val s3 = new pScoreVar(1.toString)

  val x1 = new xVar("a", s1, Set())
  val x2 = new xVar("b", s2, Set(x1))
  val x3 = new xVar("b", s3, Set(x2))

  // Discretizing scores 
  val scoreMap = List.range(0, 10).map(x => x.toString -> ((x.toDouble - 5) * 2 + 1) / 10).toMap

  val model = new TemplateModel with Parameters {
    addTemplates(
      new DotTemplateWithStatistics2[xVar, xVar] {
        val weights = Weights(new DenseTensor2(xDomain.dimensionSize, xDomain.dimensionSize))
        def unroll1(v: xVar) = {
          val factors = new ArrayBuffer[FactorType]
          v.adj.map(x => factors += Factor(v, x))
          factors
        }
        def unroll2(v2: xVar) = Nil
      },
      new Template2[xVar, pScoreVar] {
        // in TensorFamilyWithStatistics2: final override def statistics(v1:N1#Value, v2:N2#Value) = v1 outer v2
        def score(a: xVar#Value, p: pScoreVar#Value): Double = {
          if (a.category == "a") {
            return scoreMap(p.category)
          } else if (a.category == "b") {
            return 0 - scoreMap(p.category)
          } else {
            return 0
          }
        }
        def unroll1(v: xVar) = Factor(v, v.pScore)
        def unroll2(v2: pScoreVar) = Nil
      })
  }
  def main(args: Array[String]) {

    implicit val random = new scala.util.Random(2)
    // No error if I use PseudolikelihoodExample
 //   val examples =  List(x1,x2,x3).map(e => new PseudolikelihoodExample(Seq(e), model))

    val examples = List(x1, x2, x3).map(e => new LikelihoodExample(Seq(e), model, InferByBPLoopy))
    val trainer = new SampleRankTrainer(new GibbsSampler(model), new AdaGrad)
    trainer.trainFromExamples(examples)
    model.parameters.toSeq.foreach(println)
  }
}